package com.javacodegeeks.aws;

import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.AWSStaticCredentialsProvider;
import com.amazonaws.client.builder.AwsClientBuilder;
import com.amazonaws.services.dynamodbv2.AmazonDynamoDB;
import com.amazonaws.services.dynamodbv2.AmazonDynamoDBClientBuilder;
import com.amazonaws.services.dynamodbv2.local.main.ServerRunner;
import com.amazonaws.services.dynamodbv2.local.server.DynamoDBProxyServer;
import com.javacodegeeks.aws.tasks.Task;

import java.io.File;

public class App {

    public static void main(String[] args) throws Exception {
        if (args.length < 1) {
            System.err.println("Please provide the task to run.");
            System.exit(-1);
        }
        String taskName = args[0];
        try {
            Class<?> aClass = Class.forName("com.javacodegeeks.aws.tasks." + taskName);
            Object newInstance = aClass.newInstance();
            if (newInstance instanceof Task) {
                Task task = (Task) newInstance;
                String[] argsCopy = new String[args.length - 1];
                System.arraycopy(args, 1, argsCopy, 0, args.length - 1);

                AmazonDynamoDB dynamodb = null;
                String port = "8000";
                DynamoDBProxyServer server = null;
                try {
                    server = ServerRunner.createServerFromCommandLineArgs(new String[]{ "-dbPath", System.getProperty("user.dir") + File.separator + "target", "-port", port});
                    server.start();
                    dynamodb = AmazonDynamoDBClientBuilder.standard()
                            .withEndpointConfiguration(new AwsClientBuilder.EndpointConfiguration("http://localhost:" + port, "us-west-2"))
                            .withCredentials(new AWSStaticCredentialsProvider(new AWSCredentials() {
                                @Override
                                public String getAWSAccessKeyId() {
                                    return "dummy";
                                }

                                @Override
                                public String getAWSSecretKey() {
                                    return "dummy";
                                }
                            }))
                            .build();
                    task.run(argsCopy, dynamodb);
                } finally {
                    if(server != null) {
                        server.stop();
                    }
                    if(dynamodb != null) {
                        dynamodb.shutdown();
                    }
                }
            } else {
                System.err.println("Class " + aClass.getName() + " does not implement interface " + Task.class.getName() + ".");
            }
        } catch (ClassNotFoundException e) {
            System.err.println("No task with name '" + taskName + " available.");
        } catch (InstantiationException | IllegalAccessException e) {
            System.err.println("Cannot create instance of task '" + taskName + ": " + e.getLocalizedMessage());
        }
    }
}
