package com.javacodegeeks.aws.model;

import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBAttribute;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBHashKey;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBRangeKey;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBTable;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

@DynamoDBTable(tableName="TodoList")
public class TodoItem {

    private String category;
    private Date creationDate;
    private String description;
    private String status;

    public static class TodoItemBuilder {

        private String category;
        private LocalDateTime creationDate;
        private String description;
        private String status;

        public TodoItemBuilder() {

        }

        public TodoItemBuilder withCategory(String category) {
            this.category = category;
            return this;
        }

        public TodoItemBuilder withCreationDate(LocalDateTime creationDate) {
            this.creationDate = creationDate;
            return this;
        }

        public TodoItemBuilder withDescription(String desc) {
            this.description = desc;
            return this;
        }

        public TodoItemBuilder withStatus(String status) {
            this.status = status;
            return this;
        }

        public TodoItem build() {
            TodoItem item = new TodoItem();
            item.setDescription(description);
            item.setStatus(status);
            item.setCreationDate(Date.from(creationDate.atZone(ZoneId.of("UTC")).toInstant()));
            item.setCategory(category);
            return item;
        }
    }

    @DynamoDBHashKey(attributeName = "Category")
    public String getCategory() {

        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    @DynamoDBRangeKey(attributeName = "CreationDate")
    public Date getCreationDate() {
        return creationDate;
    }

    public void setCreationDate(Date creationDate) {
        this.creationDate = creationDate;
    }

    @DynamoDBAttribute(attributeName = "Description")
    public void setDescription(String description) {
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    @DynamoDBAttribute(attributeName = "Stat")
    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }
}
