package com.javacodegeeks.aws.tasks;

import com.amazonaws.event.ProgressEvent;
import com.amazonaws.event.ProgressListener;
import com.amazonaws.services.dynamodbv2.AmazonDynamoDB;
import com.amazonaws.services.dynamodbv2.document.*;
import com.amazonaws.services.dynamodbv2.document.spec.BatchGetItemSpec;
import com.amazonaws.services.dynamodbv2.document.spec.BatchWriteItemSpec;
import com.amazonaws.services.dynamodbv2.model.AttributeValue;
import com.amazonaws.services.dynamodbv2.model.BatchGetItemResult;

import java.util.List;
import java.util.Map;
import java.util.UUID;

public class BatchInsertProducts implements Task {
    @Override
    public void run(String[] args, AmazonDynamoDB dynamodb) {
        DynamoDB dynamoDB = new DynamoDB(dynamodb);

        String uuid1 = UUID.randomUUID().toString();
        String uuid2 = UUID.randomUUID().toString();
        TableWriteItems tableWriteItems = new TableWriteItems("Product")
                .withItemsToPut(new Item()
                                .withPrimaryKey("ID", uuid1)
                                .withString("Name", "Apple iPhone 7")
                                .withNumber("Price", 664.9),
                        new Item()
                                .withPrimaryKey("ID", uuid2)
                                .withString("Name", "Samsung Galaxy S8")
                                .withNumber("Price", 543.0));

        BatchWriteItemSpec spec = new BatchWriteItemSpec()
                .withProgressListener(new ProgressListener() {
                    @Override
                    public void progressChanged(ProgressEvent progressEvent) {
                        System.out.println(progressEvent.toString());
                    }
                })
                .withTableWriteItems(tableWriteItems);
        BatchWriteItemOutcome outcome = dynamoDB.batchWriteItem(spec);
        System.out.println("Unprocessed items: " + outcome.getUnprocessedItems().size());

        TableKeysAndAttributes tableKeyAndAttributes = new TableKeysAndAttributes("Product")
                .withPrimaryKeys(
                        new PrimaryKey("ID", uuid1),
                        new PrimaryKey("ID", uuid2));
        BatchGetItemSpec getSpec = new BatchGetItemSpec()
                .withProgressListener(new ProgressListener() {
                    @Override
                    public void progressChanged(ProgressEvent progressEvent) {
                        System.out.println(progressEvent.toString());
                    }
                })
                .withTableKeyAndAttributes(tableKeyAndAttributes);
        BatchGetItemOutcome batchGetItemOutcome = dynamoDB.batchGetItem(getSpec);
        System.out.println("Unprocessed keys: " + batchGetItemOutcome.getUnprocessedKeys().size());

        BatchGetItemResult batchGetItemResult = batchGetItemOutcome.getBatchGetItemResult();
        Map<String, List<Map<String, AttributeValue>>> responses = batchGetItemResult.getResponses();
        for (Map.Entry<String, List<Map<String, AttributeValue>>> entry : responses.entrySet()) {
            String tableName = entry.getKey();
            System.out.println("Table: " + tableName);
            List<Map<String, AttributeValue>> items = entry.getValue();
            for (Map<String, AttributeValue> item : items) {
                for (Map.Entry<String, AttributeValue> itemEntry : item.entrySet()) {
                    System.out.println("\t " + itemEntry.getKey() + "=" + itemEntry.getValue());
                }
            }
        }

        Table table = dynamoDB.getTable("Product");
        table.deleteItem(new PrimaryKey("ID", uuid1));
        table.deleteItem(new PrimaryKey("ID", uuid2));
    }
}
