package com.javacodegeeks.aws.tasks;

import com.amazonaws.services.dynamodbv2.AmazonDynamoDB;
import com.amazonaws.services.dynamodbv2.datamodeling.DynamoDBMapper;
import com.amazonaws.services.dynamodbv2.document.*;
import com.amazonaws.services.dynamodbv2.document.internal.PageIterable;
import com.amazonaws.services.dynamodbv2.document.spec.QuerySpec;
import com.amazonaws.services.dynamodbv2.document.utils.ValueMap;
import com.amazonaws.services.dynamodbv2.model.*;
import com.javacodegeeks.aws.model.City;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.function.Consumer;

public class Indexes implements Task {
    @Override
    public void run(String[] args, AmazonDynamoDB dynamodb) {
        createTable(dynamodb);
        populateTable(dynamodb);
        queryIndex(dynamodb);
    }

    private void queryIndex(AmazonDynamoDB dynamodb) {
        DynamoDB dynamoDB = new DynamoDB(dynamodb);
        Table table = dynamoDB.getTable("Cities");
        Index populationIndex = table.getIndex("PopulationIndex");
        ItemCollection outcome = populationIndex.query(new QuerySpec()
                .withKeyConditionExpression("Country = :country and PopulationCity > :pop")
                .withValueMap(new ValueMap()
                        .withString(":country", "China")
                        .withLong(":pop", 20000000L)));
        printResult(outcome);
    }

    private void printResult(ItemCollection items) {
        int pageNumber = 0;
        for (Object obj : items.pages()) {
            pageNumber++;
            System.out.println("Page: " + pageNumber + " #####");
            if (obj instanceof Page) {
                Page page = (Page) obj;
                Iterator iterator = page.iterator();
                while (iterator.hasNext()) {
                    Object next = iterator.next();
                    System.out.println(next);
                }
            }
        }
    }

    private void populateTable(AmazonDynamoDB dynamodb) {
        DynamoDBMapper mapper = new DynamoDBMapper(dynamodb);
        mapper.save(createCity("China", "Shanghai", 24256800, 34750000));
        mapper.save(createCity("China", "Beijing", 21516000, 24900000));
        mapper.save(createCity("Nigeria", "Lagos", 16060303, 21000000));
        mapper.save(createCity("Pakistan", "Karachi", 14910352, 14910352));
        mapper.save(createCity("Bangladesh", "Dhaka", 14543124, 14543124));
        mapper.save(createCity("China", "Guangzhou", 14043500, 44259000));
        mapper.save(createCity("Turkey", "Istanbul", 14025000, 13520000));
        mapper.save(createCity("Japan", "Tokyo", 13513734, 37843000));
        mapper.save(createCity("India", "Mumbai", 12478447, 17712000));
        mapper.save(createCity("Russia", "Moscow", 12197596, 16170000));
        mapper.save(createCity("Brazil", "São Paulo", 12038175, 21090791));
        mapper.save(createCity("India", "Delhi", 11034555, 24998000));
        mapper.save(createCity("China", "Shenzhen", 10778900, 10778900));
        mapper.save(createCity("China", "Suzhou", 10650501, 10650501));
        mapper.save(createCity("Pakistan", "Lahore", 10355000, 13569000));
        mapper.save(createCity("Egypt", "Cairo", 10230350, 18290000));
    }

    private City createCity(String country, String cityName, long populationCity, long populationMetro) {
        City city = new City();
        city.setCountry(country);
        city.setCity(cityName);
        city.setPopulationCity(populationCity);
        city.setPopulationMetropolitan(populationMetro);
        return city;
    }

    private void createTable(AmazonDynamoDB dynamodb) {
        ListTablesResult listTablesResult = dynamodb.listTables();
        List<String> tableNames = listTablesResult.getTableNames();
        for (String tableName : tableNames) {
            if ("Cities".equals(tableName)) {
                DeleteTableRequest deleteTableReq = new DeleteTableRequest().withTableName(tableName);
                dynamodb.deleteTable(deleteTableReq);
                System.out.println("Deleted table " + tableName);
            }
        }

        GlobalSecondaryIndex populationIndex = new GlobalSecondaryIndex().withIndexName("PopulationIndex")
                .withProvisionedThroughput(new ProvisionedThroughput()
                        .withReadCapacityUnits(1L)
                        .withWriteCapacityUnits(1L))
                .withKeySchema( new KeySchemaElement()
                                .withAttributeName("Country").withKeyType(KeyType.HASH),
                        new KeySchemaElement()
                                .withAttributeName("PopulationCity").withKeyType(KeyType.RANGE))
                .withProjection(new Projection()
                        .withProjectionType("KEYS_ONLY"));

        ArrayList attributeDefinitions = new ArrayList();
        attributeDefinitions.add(new AttributeDefinition()
                .withAttributeName("Country").withAttributeType("S"));
        attributeDefinitions.add(new AttributeDefinition()
                .withAttributeName("City").withAttributeType("S"));
        attributeDefinitions.add(new AttributeDefinition()
                .withAttributeName("PopulationCity").withAttributeType("N"));

        ArrayList keySchema = new ArrayList();
        keySchema.add(new KeySchemaElement()
                .withAttributeName("Country").withKeyType(KeyType.HASH));
        keySchema.add(new KeySchemaElement()
                .withAttributeName("City").withKeyType(KeyType.RANGE));

        CreateTableRequest createTableReq = new CreateTableRequest()
                .withTableName("Cities")
                .withAttributeDefinitions(attributeDefinitions)
                .withKeySchema(keySchema)
                .withProvisionedThroughput(new ProvisionedThroughput()
                        .withReadCapacityUnits(10L)
                        .withWriteCapacityUnits(5L))
                .withGlobalSecondaryIndexes(populationIndex);
        CreateTableResult result = dynamodb.createTable(createTableReq);
        System.out.println("Created table " + result.toString());

        DynamoDB dynamoDB = new DynamoDB(dynamodb);
        Table table = dynamoDB.getTable("Cities");
        try {
            table.waitForActive();
        } catch (InterruptedException e) {
            System.err.println("Waiting was interrupted: " + e.getLocalizedMessage());
        }
    }
}