package com.javacodegeeks.aws.tasks;

import com.amazonaws.services.dynamodbv2.AmazonDynamoDB;
import com.amazonaws.services.dynamodbv2.document.DynamoDB;
import com.amazonaws.services.dynamodbv2.document.Item;
import com.amazonaws.services.dynamodbv2.document.PrimaryKey;
import com.amazonaws.services.dynamodbv2.document.Table;
import com.amazonaws.services.dynamodbv2.document.spec.UpdateItemSpec;
import com.amazonaws.services.dynamodbv2.document.utils.NameMap;
import com.amazonaws.services.dynamodbv2.document.utils.ValueMap;

import java.util.UUID;

public class InsertProducts implements Task {
    @Override
    public void run(String[] args, AmazonDynamoDB dynamodb) {
        DynamoDB dynamoDB = new DynamoDB(dynamodb);
        Table table = dynamoDB.getTable("Product");

        String uuid1 = UUID.randomUUID().toString();
        Item item = new Item()
                .withPrimaryKey("ID", uuid1)
                .withString("Name", "Apple iPhone 7")
                .withNumber("Price", 664.9);
        table.putItem(item);

        String uuid2 = UUID.randomUUID().toString();
        item = new Item()
                .withPrimaryKey("ID", uuid2)
                .withString("Name", "Samsung Galaxy S8")
                .withNumber("Price", 543.0);
        table.putItem(item);

        item = table.getItem("ID", uuid1);
        System.out.println("Retrieved item with UUID " + uuid1 + ": name="
                + item.getString("Name") + "; price=" + item.getString("Price"));
        item = table.getItem("ID", uuid2);
        System.out.println("Retrieved item with UUID " + uuid2 + ": name="
                + item.getString("Name") + "; price=" + item.getString("Price"));

        UpdateItemSpec updateSpec = new UpdateItemSpec()
                .withPrimaryKey(new PrimaryKey("ID", uuid1))
                .withUpdateExpression("set Price = :priceNew")
                .withConditionExpression("Price = :priceOld")
                .withValueMap(new ValueMap()
                                .withNumber(":priceOld", 664.9)
                                .withNumber(":priceNew", 645.9)
                );
        table.updateItem(updateSpec);

        item = table.getItem("ID", uuid1);
        System.out.println("Retrieved item with UUID " + uuid1 + ": name="
                + item.getString("Name") + "; price=" + item.getString("Price"));

        table.deleteItem(new PrimaryKey("ID", uuid1));
        table.deleteItem(new PrimaryKey("ID", uuid2));

        System.out.println(table.describe());
    }
}
